/**
 * 
 */
package sim.application;

import sim.exception.SConstructorException;
import sim.exception.SNoImplementationException;
import sim.math.SVector3d;
import sim.physics.SElectrostatics;
import sim.physics.SParticle;
import sim.physics.SParticlesSystem;

/**
 * L'application <b>SIMCoulomb</b> reprsente un calculateur de force lectrique.
 * Le calcul peut tre ralis  partir de :
 * <ul> - un fichier texte o sont prcises les proprits des particules charges </ul>
 * <ul> - la spcfication d'une gomtrie (particules ponctuelles, TRIUC, PPIUC) </ul> 
 * 
 * @author Simon Vzina
 * @since 2017-05-30
 * @version 2019-09-05 (Version labo v1.1.4 : La loi de Coulomb)
 */
public class SIMCoulomb {

  /**
   * La constante <b>FILE_NAME</b> correspond au nom du fichier en lecture utilis pour dfinir un systme de particule analys lors de la 3ime loi de Newton.
   */
  private static final String FILE_NAME = "systeme_particule.txt";
  
  /**
   * La mthode main.
   * 
   * @param args
   */
  public static void main(String[] args)
  {
    // Affichage rsultat : Mise en situation - Question 1. (prlaboratoire)
    miseEnSituationQ1();
    
    // Affichage rsultat : Mise en situation - Question 2. (prlaboratoire)
    miseEnSituationQ2();
    
    // Affichage rsultat : Mise en situation - Question 3. et 4. (prlaboratoire)
    miseEnSituationQ3Q4();
    
    // Affichage rsultat : Mise en situation - Question 5. (prlaboratoire)
    miseEnSituationQ5();
    
    // Vrification de la 3ime loi de Newton avec la lecture d'un fichier contenant la description du systme de particules.   
    newtonThirdLaw(FILE_NAME);
            
    //------------------------------------------------------------------------
    // Effectuer la comparaison des forces lectriques (Sphre, TRIUC, PPIUC).
    //------------------------------------------------------------------------
    double Q = 1.0;   // en Coulomb
    double q = 1.0;   // en Coulomb
    double L = 2;     // en mtre
    double d = 0.01;  // en mtre
    int N = 10;       // nombre d'lment discret par dimension
    
    comparaisonDistribution(Q, q, L, d, N);   // VOUS DEVEZ CHANGER les valeurs pour effectuer d'autres comparaisons.  
  }

  //------------------------
  // MTHODES UTILITAIRES //
  //------------------------
  
  /**
   * Mthode ralisant l'affichage de la mise en situation - Question 1. (prlaboratoire)
   */
  public static void miseEnSituationQ1()
  {
    System.out.println("Rsultat : Mise en situation - Question 1. (prlaboratoire)");
    
    //-----------------------------------
    //  VOUS DE MOFIFIER LE PROGRAMME. 
    //-----------------------------------
    
    
  }
  
  /**
   * Mthode ralisant l'affichage de la mise en situation - Question 2. (prlaboratoire)
   */
  public static void miseEnSituationQ2()
  {
    System.out.println("Rsultat : Mise en situation - Question 2. (prlaboratoire)");
    
    //-----------------------------------
    //  VOUS DE MOFIFIER LE PROGRAMME. 
    //-----------------------------------
    
    
  }
  
  /**
   * Mthode ralisant l'affichage de la mise en situation - Question 3. et 4. (prlaboratoire)
   */
  public static void miseEnSituationQ3Q4()
  {
    System.out.println("Rsultat : Mise en situation - Question 3. et 4. (prlaboratoire)");
    
    //-----------------------------------
    //  VOUS DE MOFIFIER LE PROGRAMME. 
    //-----------------------------------
    
   
  }
  
  /**
   * Mthode ralisant l'affichage de la mise en situation - Question 5. (prlaboratoire)
   */
  public static void miseEnSituationQ5()
  {
    System.out.println("Rsultat : Mise en situation - Question 5. (prlaboratoire)");
    
    //-----------------------------------
    //  VOUS DE MOFIFIER LE PROGRAMME. 
    //-----------------------------------
    
    
  }
  
  /**
   * Mthode pour effectuer la vrification de la 3ime loi de Newton.
   * 
   * @param file_name Le nom du fichier en lecture pour dfinir le systme de particule.
   */
  public static void newtonThirdLaw(String file_name)
  {
    try{
    
      // Construction d'un systme de particules  partir du nom d'un fichier.
      SParticlesSystem system = new SParticlesSystem(file_name);
      
      // Calcul pour valuer la 3ime loi de Newton applique au systme de particule.
      SVector3d sum = SVector3d.ZERO;
    
      for(int i = 0; i < system.getNbParticles(); i++)
        sum = sum.add(system.evaluateForce(i));
    
      // Affichage du rsultat.
      System.out.println("Nombre de particules dans le systme est N = " + system.getNbParticles());
      System.out.println("Liste des particules du systme : ");
      System.out.println(system);
      System.out.println("La somme des forces du systme : F = " + sum + " N");
      System.out.println();
      
    }catch(SConstructorException e){
      System.out.println("La 3ime loi de Newton n'a pas pu tre value, car le fichier '" + file_name + "' n'a pas t trouv.");
    }catch(SNoImplementationException e){
      System.out.println("La 3ime loi de Newton n'a pas pu tre value, car une mthode ncessaire au calcul n'a pas t implmente.");
    }
    
  }
 
  /**
   * <p>
   * Mthode pour faire la comparaison de la force lectrique qu'applique diffrents systmes sur une particule charge.
   * Les systmes en comparaison sont :
   * <ul> - La charge ponctuelle. </ul>
   * <ul> - La tige infinie uniformment charge (TRIUC) avec N lments discrets. </ul>
   * <ul> - La plaque plane infinie uniformment charge (PPIUC) avec N lments discrets. </ul>
   * </p>
   * 
   * @param Q La charge qui applique la force en coulombs (C).
   * @param q La charge qui subit la force lectrique en coulombs (C).
   * @param L La taille de la gomtrie en mtres (m).
   * @param d La distance entre la distribution de charge et la charge qui subit la force en mtres (m).
   * @param N Le nombre d'lments discrets dans les gomtries.
   */
  public static void comparaisonDistribution(double Q, double q, double L, double d, int N)
  {
     try{
    
      // 1er systme : La charge ponctuelle.
      SParticlesSystem sys1 = new SParticlesSystem();
      sys1.add(new SParticle(Q, new SVector3d(0.0, 0.0, 0.0)));
      sys1.add(new SParticle(q, new SVector3d(0.0, 0.0, d)));
      
      // 2e systme : La TRIUC. 
      SParticlesSystem sys2 = SParticlesSystem.buildLineX(Q, 2.0, N);
      sys2.add(new SParticle(q, new SVector3d(L/2.0, 0.0, d)));
          
      // 3e systme : La PPIUC.
      SParticlesSystem sys3 = SParticlesSystem.buildSquareXY(Q, 2.0, N);
      sys3.add(new SParticle(q, new SVector3d(L/2.0, L/2.0, d)));
      
      // Calcul des forces lectriques des systmes sur une charge ponctuelle.
      // P.S. La charque q est situe  la fin du systme (en index = N-1).
      SVector3d force_ponctuelle = sys1.evaluateForce(sys1.getNbParticles()-1);
      SVector3d force_TRIUC = sys2.evaluateForce(sys2.getNbParticles()-1);
      SVector3d force_PPIUC = sys3.evaluateForce(sys3.getNbParticles()-1);
      
      // Affichage des rsultats.
      System.out.println("Comparaison des distributions de charges :");
      System.out.println("------------------------------------------");
      System.out.println("Q = " + Q + " C , q = " + q + " C");
      System.out.println("L = " + L + " m , N = " + N + " particules");
      System.out.println("d = " + d + " m");
      
      System.out.println("Force applique par une charge PONCTUELLE sur une particule : F = " + force_ponctuelle + " N");
      System.out.println("Force applique par une TRIUC sur une particule             : F = " + force_TRIUC + " N");
      System.out.println("Force applique par une PPIUC sur une particule             : F = " + force_PPIUC + " N");
    
    }catch(SNoImplementationException e){ 
      System.out.println("La comparaison des distributions des charges n'est pas possible, car certaines mthodes ncessaires au calcul n'ont pas t implmentes.");
    }
  }
  
}
