package sim.math.field;

import sim.math.SVector3d;

/**
 * La classe <b>SUndefinedElectricFieldException</b> reprsente une exception lorsqu'un champ lectrique n'est pas dfini  l'endroit calcul.
 * 
 * @author Simon Vezina
 * @since 2020-06-16
 * @version 2022-04-08
 */
public class SUndefinedFieldException extends RuntimeException {

  /**
   * 
   */
  private static final long serialVersionUID = 985478302009607108L;

  /**
   * La constante <b>DEFAULT_MESSAGE_WITH_POSITION</b> reprsente le message d'erreur par dfaut avec l'usage d'une coordonne.
   */
  private static final String DEFAULT_MESSAGE_WITH_POSITION = "Le champ lectrique n'est pas dfini  cette position : ";
  
  /**
   * La constante <b>DEFAULT_MESSAGE</b> reprsente le message d'erreur par dfaut sans l'usage d'une coordonne.
   */
  private static final String DEFAULT_MESSAGE = "Le champ lectique n'est pas dfini.";
  
  /**
   * Constructeur d'une exception stipulant qu'un champ ne peut pas tre valu.
   */
  public SUndefinedFieldException()
  {
    super(DEFAULT_MESSAGE);
  }
  
  /**
   * Constructeur d'une exception stipulant qu'un champ ne peut pas tre valu  une certaine coordonne.
   * 
   * @param message Le message dtaillant pourquoi le champ ne peut pas tre dfini.
   * @param r La position o le champ n'est pas dfini.
   */
  public SUndefinedFieldException(String message, SVector3d r)
  {
    super(message + ". " + DEFAULT_MESSAGE_WITH_POSITION + r.toString());
  }
  
  /**
   * Constructeur d'une exception stipulant qu'un champ ne peut par tre valu  une certaine coordonne.
   * 
   * @param r La position o le champ n'est pas dfini.
   */
  public SUndefinedFieldException(SVector3d r)
  {
    super(DEFAULT_MESSAGE_WITH_POSITION + r.toString());
  }
    
}
